import {
  Category,
  Expression,
  ExpressionSet,
  ExpressionToCategory,
  ExpressionToExpressionSet,
} from "../model";

interface RawExpressionDataItem {
  prompt: string;
  description: string;
  category: string;
  expression_set: string;
}

interface RawExpressionSetItem {
  name: string;
  description: string;
}

const RawExpressionData: RawExpressionDataItem[] = [
  ["koira", "noun", "daily", "dog, dog paddle (swimming stroke)"],
  ["liite", "noun", "daily", "attachment, appendix, affix, supplement"],
  ["havaita", "verb", "daily", "to observe, to detect, to perceive"],
  ["tukea", "verb", "daily", "to support, finance, sponsor, substantiate"],
].map(([prompt, category, expression_set, description]) => ({
  prompt,
  description,
  category,
  expression_set,
}));

const RawExpressionSetData: RawExpressionSetItem[] = [
  ["daily", "New expressions and poorly remembered expressions"],
  ["weekly", "Expressions to be reviewed at the end of the week"],
  ["monthly", "Expressions to be reviewed at the end of the month"],
  ["yearly", "Expressions to be reviewed at the end of the year"],
  ["ancient", "Expressions here should be memorized by now"],
].map(([name, description]) => ({ name, description }));

interface MockData {
  // Tables
  categories: Category[];
  expressions: Expression[];
  expression_sets: ExpressionSet[];

  // Relationships
  expression_to_category: ExpressionToCategory[];
  expression_to_expression_set: ExpressionToExpressionSet[];
}

export function parseRawData(
  raw_expression_data: RawExpressionDataItem[],
  raw_expression_set_data: RawExpressionSetItem[]
): MockData {
  const category_names = new Set(
    raw_expression_data.map((item) => item.category)
  );
  const categories: Category[] = Array.from(category_names).map(
    (name, index) => ({
      id: index + 1,
      name,
      description: name,
    })
  );

  const expressions: Expression[] = raw_expression_data.map(
    ({ prompt, description }, index) => ({
      id: index + 1,
      prompt,
      description,
    })
  );

  const expression_sets: ExpressionSet[] = raw_expression_set_data.map(
    ({ name, description }, index) => ({
      id: index + 1,
      name,
      description,
    })
  );

  const expression_to_category: ExpressionToCategory[] =
    raw_expression_data.map((item, index) =>
      matchExpressionAndCategory(index + 1, item, categories)
    );

  const expression_to_expression_set: ExpressionToExpressionSet[] =
    raw_expression_data.map((item, index) =>
      matchExpressionAndExpressionSet(index + 1, item, expression_sets)
    );

  return {
    expressions,
    expression_sets,
    categories,
    expression_to_category,
    expression_to_expression_set,
  };
}

function matchExpressionAndCategory(
  expression_id: number,
  { category }: RawExpressionDataItem,
  categories: Category[]
): ExpressionToCategory {
  const category_id = categories.find(({ name }) => name === category)?.id || 0;
  return {
    category_id,
    expression_id,
  };
}

function matchExpressionAndExpressionSet(
  expression_id: number,
  { expression_set }: RawExpressionDataItem,
  expression_sets: ExpressionSet[]
): ExpressionToExpressionSet {
  const expression_set_id =
    expression_sets.find(({ name }) => name === expression_set)?.id || 0;
  return {
    expression_id,
    expression_set_id,
  };
}

export const MockData = parseRawData(RawExpressionData, RawExpressionSetData);
